@@ -36,6 +36,7 @@
 #include <tbb/parallel_for.h>
 #include <tbb/parallel_for_each.h>
 #include <tbb/blocked_range.h>
+#include <tbb/task_arena.h>
 #include <inja/inja.hpp>
 
 #include "datainjectpkg.h"
@@ -44,6 +45,7 @@
 #include "logging.h"
 #include "result.h"
 #include "utils.h"
+#include "yaml-utils.h"
 #include "zarchive.h"
 #include "backends/interfaces.h"
 
@@ -63,6 +65,13 @@ Engine::Engine()
     : m_conf(&Config::get()),
       m_forced(false)
 {
+    // Configure a TBB task arena to limit parallelism a little (use half the available CPU cores, or at least 6
+    // threads) This avoids having too many parallel downloads on high-core-count machines, and also leaves some room
+    // for additional parallelism of the used libraries, e.g. for image processing.
+    const auto numCPU = std::thread::hardware_concurrency();
+    const auto maxThreads = std::max(numCPU > 6 ? 6L : numCPU, std::lround(numCPU * 0.60));
+    m_taskArena = std::make_unique<tbb::task_arena>(maxThreads);
+
     // Select backend
     switch (m_conf->backend) {
     case Backend::Dummy:
@@ -123,6 +132,22 @@ void Engine::logVersionInfo()
     logInfo("AppStream Generator {}, AS: {}{}", ASGEN_VERSION, asVersion, backendInfo);
 }
 
+void Engine::checkLibfyamlVersion()
+{
+    const auto version = Yaml::libfyamlVersion();
+    if (version.empty()) {
+        logWarning("Unable to determine libfyaml version! Please ensure you have at least version 0.9 installed!");
+        return;
+    }
+
+    if (as_vercmp_simple(version.c_str(), "0.9") < 0) {
+        throw std::runtime_error(std::format(
+            "libfyaml version {} is too old, at least version 0.9 is required! Please upgrade libfyaml, as versions <= "
+            "0.8 have known bugs in unicode handling and may generate corrupted UTF-8 data.",
+            version));
+    }
+}
+
 void Engine::processPackages(
     const std::vector<std::shared_ptr<Package>> &pkgs,
     std::shared_ptr<IconHandler> iconh,
@@ -130,6 +155,13 @@ void Engine::processPackages(
 {
     g_autoptr(AsgLocaleUnit) localeUnit = asg_locale_unit_new(m_cstore, pkgs);
 
+    {
+        g_autoptr(GError) error = NULL;
+        if (!asc_unit_open(ASC_UNIT(localeUnit), &error))
+            throw std::runtime_error(
+                std::format("Failed to open locale unit: {}", error ? error->message : "Unknown error"));
+    }
+
     const auto numProcessors = std::thread::hardware_concurrency();
     std::size_t chunkSize = pkgs.size() / numProcessors / 10;
     if (chunkSize > 100)
@@ -137,32 +169,43 @@ void Engine::processPackages(
     if (chunkSize <= 10)
         chunkSize = 10;
 
-    logDebug("Analyzing {} packages in batches of {}", pkgs.size(), chunkSize);
+    logDebug(
+        "Analyzing {} packages in batches of {} with {} parallel tasks",
+        pkgs.size(),
+        chunkSize,
+        m_taskArena->max_concurrency());
 
-    tbb::parallel_for(
-        tbb::blocked_range<std::size_t>(0, pkgs.size(), chunkSize), [&](const tbb::blocked_range<std::size_t> &range) {
-            auto mde = std::make_unique<DataExtractor>(m_dstore, iconh, localeUnit, injMods);
+    m_taskArena->execute([&] {
+        tbb::parallel_for(
+            tbb::blocked_range<std::size_t>(0, pkgs.size(), chunkSize),
+            [&](const tbb::blocked_range<std::size_t> &range) {
+                auto mde = std::make_unique<DataExtractor>(m_dstore, iconh, localeUnit, injMods);
 
-            for (std::size_t i = range.begin(); i != range.end(); ++i) {
-                auto pkg = pkgs[i];
-                const auto &pkid = pkg->id();
+                for (std::size_t i = range.begin(); i != range.end(); ++i) {
+                    auto pkg = pkgs[i];
+                    const auto &pkid = pkg->id();
 
-                if (m_dstore->packageExists(pkid))
-                    continue;
+                    if (m_dstore->packageExists(pkid))
+                        continue;
 
-                auto res = mde->processPackage(pkg);
-                {
-                    std::lock_guard<std::mutex> lock(m_mutex);
-                    // Write resulting data into the database
-                    m_dstore->addGeneratorResult(m_conf->metadataType, res);
+                    auto res = mde->processPackage(pkg);
+                    {
+                        std::lock_guard<std::mutex> lock(m_mutex);
+                        // Write resulting data into the database
+                        m_dstore->addGeneratorResult(m_conf->metadataType, res);
+                    }
+
+                    logInfo(
+                        "Processed {}, components: {}, hints: {}", res.pkid(), res.componentsCount(), res.hintsCount());
+
+                    // We don't need content data from this package anymore
+                    pkg->finish();
                 }
+            });
+    });
 
-                logInfo("Processed {}, components: {}, hints: {}", res.pkid(), res.componentsCount(), res.hintsCount());
-
-                // We don't need content data from this package anymore
-                pkg->finish();
-            }
-        });
+    // not strictly necessary, but let's close the unit explicitly
+    asc_unit_close(ASC_UNIT(localeUnit));
 }
 
 // Helper to check if a package may contain interesting metadata
@@ -215,7 +258,11 @@ bool Engine::seedContentsData(
     if (workUnitSize > 30)
         workUnitSize = 30;
 
-    logDebug("Scanning {} packages, work unit size {}", pkgs.size(), workUnitSize);
+    logDebug(
+        "Scanning {} packages, work unit size: {}, parallel tasks: {}",
+        pkgs.size(),
+        workUnitSize,
+        m_taskArena->max_concurrency());
 
     // Check if the index has changed data, skip the update if there's nothing new
     if (pkgs.empty() && !m_pkgIndex->hasChanges(m_dstore, suite.name, section, arch) && !m_forced) {
@@ -237,68 +284,72 @@ bool Engine::seedContentsData(
         logInfo("Scanning new packages for base suite {}/{} [{}]", suite.baseSuite, section, arch);
         auto baseSuitePkgs = m_pkgIndex->packagesFor(suite.baseSuite, section, arch);
 
-        tbb::parallel_for(
-            tbb::blocked_range<std::size_t>(0, baseSuitePkgs.size(), workUnitSize),
-            [&](const tbb::blocked_range<std::size_t> &range) {
-                for (size_t i = range.begin(); i != range.end(); ++i) {
-                    auto pkg = baseSuitePkgs[i];
-                    const auto &pkid = pkg->id();
+        m_taskArena->execute([&] {
+            tbb::parallel_for(
+                tbb::blocked_range<std::size_t>(0, baseSuitePkgs.size(), workUnitSize),
+                [&](const tbb::blocked_range<std::size_t> &range) {
+                    for (size_t i = range.begin(); i != range.end(); ++i) {
+                        auto pkg = baseSuitePkgs[i];
+                        const auto &pkid = pkg->id();
 
-                    if (!m_cstore->packageExists(pkid)) {
-                        m_cstore->addContents(pkid, pkg->contents());
-                        logInfo("Scanned {} for base suite.", pkid);
+                        if (!m_cstore->packageExists(pkid)) {
+                            m_cstore->addContents(pkid, pkg->contents());
+                            logInfo("Scanned {} for base suite.", pkid);
+                        }
+
+                        // Chances are that we might never want to extract data from these packages, so remove their
+                        // temporary data for now - we can reopen the packages later if we actually need them.
+                        pkg->cleanupTemp();
                     }
-
-                    // Chances are that we might never want to extract data from these packages, so remove their
-                    // temporary data for now - we can reopen the packages later if we actually need them.
-                    pkg->cleanupTemp();
-                }
-            });
+                });
+        });
     }
 
     // And then scan the suite itself - here packages can be 'interesting'
     // in that they might end up in the output.
-    tbb::parallel_for(
-        tbb::blocked_range<std::size_t>(0, packagesToProcess.size(), workUnitSize),
-        [&](const tbb::blocked_range<std::size_t> &range) {
-            for (std::size_t i = range.begin(); i != range.end(); ++i) {
-                auto pkg = packagesToProcess[i];
-                const auto &pkid = pkg->id();
+    m_taskArena->execute([&] {
+        tbb::parallel_for(
+            tbb::blocked_range<std::size_t>(0, packagesToProcess.size(), workUnitSize),
+            [&](const tbb::blocked_range<std::size_t> &range) {
+                for (std::size_t i = range.begin(); i != range.end(); ++i) {
+                    auto pkg = packagesToProcess[i];
+                    const auto &pkid = pkg->id();
 
-                std::vector<std::string> contents;
-                if (m_cstore->packageExists(pkid)) {
-                    if (m_dstore->packageExists(pkid)) {
-                        // TODO: Unfortunately, packages can move between suites without changing their ID.
-                        // This means as soon as we have an interesting package, even if we already processed it,
-                        // we need to regenerate the output metadata.
-                        // For that to happen, we set interestingFound to true here. Later, a more elegant solution
-                        // would be desirable here, ideally one which doesn't force us to track which package is
-                        // in which suite as well.
-                        if (!m_dstore->isIgnored(pkid))
-                            interestingFound.store(true);
-                        return;
+                    std::vector<std::string> contents;
+                    if (m_cstore->packageExists(pkid)) {
+                        if (m_dstore->packageExists(pkid)) {
+                            // TODO: Unfortunately, packages can move between suites without changing their ID.
+                            // This means as soon as we have an interesting package, even if we already processed it,
+                            // we need to regenerate the output metadata.
+                            // For that to happen, we set interestingFound to true here. Later, a more elegant solution
+                            // would be desirable here, ideally one which doesn't force us to track which package is
+                            // in which suite as well.
+                            if (!m_dstore->isIgnored(pkid))
+                                interestingFound.store(true);
+                            continue;
+                        }
+                        // We will complement the main database with ignore data, in case it
+                        // went missing.
+                        contents = m_cstore->getContents(pkid);
+                    } else {
+                        // Add contents to the index
+                        contents = pkg->contents();
+                        m_cstore->addContents(pkid, contents);
                     }
-                    // We will complement the main database with ignore data, in case it
-                    // went missing.
-                    contents = m_cstore->getContents(pkid);
-                } else {
-                    // Add contents to the index
-                    contents = pkg->contents();
-                    m_cstore->addContents(pkid, contents);
-                }
 
-                // Check if we can already mark this package as ignored, and print some log messages
-                if (!packageIsInteresting(pkg)) {
-                    m_dstore->setPackageIgnore(pkid);
-                    logInfo("Scanned {}, no interesting files found.", pkid);
-                    // We won't use this anymore
-                    pkg->finish();
-                } else {
-                    logInfo("Scanned {}, could be interesting.", pkid);
-                    interestingFound.store(true);
+                    // Check if we can already mark this package as ignored, and print some log messages
+                    if (!packageIsInteresting(pkg)) {
+                        m_dstore->setPackageIgnore(pkid);
+                        logInfo("Scanned {}, no interesting files found.", pkid);
+                        // We won't use this anymore
+                        pkg->finish();
+                    } else {
+                        logInfo("Scanned {}, could be interesting.", pkid);
+                        interestingFound.store(true);
+                    }
                 }
-            }
-        });
+            });
+    });
 
     // Ensure the contents store is in a consistent state on disk,
     // since it might be accessed from other threads after this function
@@ -348,7 +399,7 @@ std::string Engine::getMetadataHead(const Suite &suite, const std::string &secti
         if (suite.dataPriority != 0)
             head += std::format("\nPriority: {}", suite.dataPriority);
         if (m_conf->feature.metadataTimestamps)
-            head += std::format("\nTime: {}", timeNowIso8601);
+            head += std::format("\nTime: '{}'", timeNowIso8601);
     }
 
     return head;
@@ -405,7 +456,7 @@ void Engine::exportMetadata(
             if (!mres.empty()) {
                 std::lock_guard<std::mutex> lock(exportMutex);
                 for (const auto &md : mres)
-                    mdataFile << std::format("{}\n", md);
+                    mdataFile << md << "\n";
             }
 
             for (const auto &gcid : gcids) {
@@ -799,6 +850,9 @@ bool Engine::processFile(
         return false;
     }
 
+    // ensure our libfyaml version is new enough
+    checkLibfyamlVersion();
+
     std::unordered_map<std::string, std::vector<std::shared_ptr<Package>>> pkgByArch;
     for (const auto &fname : files) {
         auto pkg = m_pkgIndex->packageForFile(fname, suiteName, sectionName);
@@ -835,6 +889,44 @@ bool Engine::processFile(
     return true;
 }
 
+void Engine::run()
+{
+    logVersionInfo();
+    checkLibfyamlVersion();
+
+    bool dataChanged = false;
+    auto reportgen = std::make_shared<ReportGenerator>(m_dstore.get());
+
+    for (const auto &suite : m_conf->suites) {
+        if (suite.isImmutable) {
+            logDebug("Skipping immutable suite: {}", suite.name);
+            continue;
+        }
+
+        // The `checkSuiteUsable` method will print an error
+        // message in case the suite isn't usable.
+        auto suiteCheck = checkSuiteUsable(suite.name);
+        if (!suiteCheck.suiteUsable)
+            continue;
+
+        printHeaderBox(std::format("Processing: {}", suite.name));
+
+        // process the suite
+        for (const auto &section : suite.sections) {
+            printSectionBox(std::format("{}: {}", suite.name, section));
+            const bool ret = processSuiteSection(suite, section, reportgen);
+            if (ret)
+                dataChanged = true;
+        }
+    }
+
+    // Render index pages & statistics
+    printHeaderBox("Updating Global Data");
+    reportgen->updateIndexPages();
+    if (dataChanged)
+        reportgen->exportStatistics();
+}
+
 void Engine::run(const std::string &suiteName)
 {
     // Fetch suite and exit in case we can't write to it.
@@ -846,6 +938,7 @@ void Engine::run(const std::string &suiteName)
     auto suite = suiteCheck.suite;
 
     logVersionInfo();
+    checkLibfyamlVersion();
 
     auto reportgen = std::make_shared<ReportGenerator>(m_dstore.get());
 
@@ -873,6 +966,7 @@ void Engine::run(const std::string &suiteName, const std::string &sectionName)
     auto suite = suiteCheck.suite;
 
     logVersionInfo();
+    checkLibfyamlVersion();
 
     bool sectionValid = false;
     for (const auto &section : suite.sections) {
@@ -1135,11 +1229,15 @@ void Engine::forgetPackage(const std::string &identifier)
 {
     const auto slashCount = std::count(identifier.begin(), identifier.end(), '/');
 
+    std::unordered_set<std::string> affectedGCIDs;
     if (slashCount == 2) {
         // We have a package-id, so we can do a targeted remove
         const auto pkid = identifier;
         logDebug("Considering {} to be a package-id.", pkid);
 
+        const auto gcids = m_dstore->getGCIDsForPackage(pkid);
+        affectedGCIDs.insert(gcids.begin(), gcids.end());
+
         if (m_cstore->packageExists(pkid))
             m_cstore->removePackage(pkid);
         if (m_dstore->packageExists(pkid))
@@ -1148,6 +1246,9 @@ void Engine::forgetPackage(const std::string &identifier)
     } else {
         auto pkids = m_dstore->getPkidsMatching(identifier);
         for (const auto &pkid : pkids) {
+            const auto gcids = m_dstore->getGCIDsForPackage(pkid);
+            affectedGCIDs.insert(gcids.begin(), gcids.end());
+
             m_dstore->removePackage(pkid);
             if (m_cstore->packageExists(pkid))
                 m_cstore->removePackage(pkid);
@@ -1157,6 +1258,20 @@ void Engine::forgetPackage(const std::string &identifier)
 
     // Remove orphaned data and media
     m_dstore->cleanupCruft();
+
+    // Report if data is kept because packages keep the GCIDs around
+    std::cout << "Packages still using the same data from the removed package(s):" << std::endl;
+    auto pkgsForGCIDs = m_dstore->getPackagesForGCIDs(affectedGCIDs);
+    if (pkgsForGCIDs.empty()) {
+        std::cout << "  - None" << std::endl;
+    } else {
+        for (const auto &it : pkgsForGCIDs) {
+            std::cout << "  - " << it.first << ": ";
+            for (const auto &gcid : it.second)
+                std::cout << gcid << " ";
+            std::cout << std::endl;
+        }
+    }
 }
 
 bool Engine::printPackageInfo(const std::string &identifier)
